
/*
 * fmath.c -- sin, cos, tan, acos, asin, atan, dtor, rtod, exp, log, sqrt
 */

#include "../h/config.h"
#include "../h/rt.h"
#include "rproto.h"
#include <math.h>

/*
 * The following code is operating-system dependent [@fmath.01].  Include
 *  system-dependent files.
 */

#if PORT
   /* probably #include <errno.h> */
#endif					/* PORT */

#if AMIGA || HIGHC_386 || MACINTOSH || MVS || UNIX || VM || VMS
#include <errno.h>
#endif					/* AMIGA || HIGHC_386 ... */

#if ATARI_ST
#if LATTICE
#include <error.h>
#else					/* LATTICE */
#include <errno.h>
#endif					/* LATTICE */
#endif					/* ATARI_ST */

#if MSDOS
#if !MWC
#include <errno.h>
#endif					/* !MWC */
#endif					/* MSDOS */

/*
 * End of operating-system specific code.
 */

#ifdef PreProcess
/* include(../M4/fncs.m4) /* */
/* */
#endif					/* PreProcess */

#ifdef MathFncs
/*
 * The following code is operating-system dependent [@fxmath.01].
 *  declare errno if not done already.
 */

#if PORT
#endif					/* PORT */

#if AMIGA
#endif					/* AMIGA */

#if ATARI_ST
#endif					/* ATARI_ST */

#if HIGHC_386
#endif					/* HIGHC_386 */

#if MACINTOSH
#endif					/* MACINTOSH */

#if MSDOS || OS2
#if MICROSOFT
int errno;
#endif					/* MICROSOFT */
#endif					/* MSDOS || OS2 */

#if MVS || VM
#ifdef SASC
#include <lcmath.h>
#define PI M_PI
#endif					/* SASC */
#endif					/* MVS || VM */

#if UNIX
int errno;
#endif					/* UNIX */

#if VMS
#endif					/* VMS */

/*
 * End of operating-system specific code.
 */

#ifndef PI
#define PI 3.14159
#endif					/* PI */

/*
 * sin(x), x in radians
 */

FncDcl(sin,1)
   {
   int t;
   union numeric r;
   double sin();

   if ((t = cvreal(&Arg1, (double *)&r)) == CvtFail) 
     RunErr(102, &Arg1);
   if (mkreal(sin(r.real),&Arg0) == Error) 
      RunErr(0, NULL);
   Return;
   }

/*
 * cos(x), x in radians
 */

FncDcl(cos,1)
   {
   int t;
   union numeric r;
   double cos();

   if ((t = cvreal(&Arg1, (double *)&r)) == CvtFail) 
      RunErr(102, &Arg1);
   if (mkreal(cos(r.real),&Arg0) == Error) 
      RunErr(0, NULL);
   Return;
   }

/*
 * tan(x), x in radians
 */

FncDcl(tan,1)
   {
   int t;
   double y;
   union numeric r;
   double tan();

   if ((t = cvreal(&Arg1, (double *)&r)) == CvtFail) 
      RunErr(102, &Arg1);
   errno = 0;
   y = tan(r.real);
   if (errno == ERANGE) 
      RunErr(-204, NULL);
   if (mkreal(y,&Arg0) == Error) 
      RunErr(0, NULL);
   Return;
   }

/*
 * acos(x), x in radians
 */
FncDcl(acos,1)
   {
   int t;
   double y;
   union numeric r;
   double acos();

   if ((t = cvreal(&Arg1, (double *)&r)) == CvtFail) 
      RunErr(102, &Arg1);
   errno = 0;
   y = acos(r.real);
   if (errno == EDOM) 
      RunErr(-205, NULL);
   if (mkreal(y,&Arg0) == Error) 
      RunErr(0, NULL);
   Return;
   }

/*
 * asin(x), x in radians
 */
FncDcl(asin,1)
   {
   int t;
   double y;
   union numeric r;
   double asin();

   if ((t = cvreal(&Arg1, (double *)&r)) == CvtFail) 
      RunErr(102, &Arg1);
   errno = 0;
   y = asin(r.real);
   if (errno == EDOM) 
      RunErr(-205, NULL);
   if (mkreal(y,&Arg0) == Error) 
      RunErr(0, NULL);
   Return;
   }

/*
 * atan(x,y) -- x,y  in radians; if y is present, produces atan2(x,y).
 */
FncDcl(atan,2)
   {
   int t;
   union numeric r1, r2;
   double atan(), atan2();

   if ((t = cvreal(&Arg1, (double *)&r1)) == CvtFail) 
      RunErr(102, &Arg1);
   if (ChkNull(Arg2)) {
      if (mkreal(atan(r1.real),&Arg0) == Error) 
         RunErr(0, NULL);
      }
   else {
      if ((t = cvreal(&Arg2, (double *)&r2)) == CvtFail) 
         RunErr(102, &Arg2);
      if (mkreal(atan2(r1.real,r2.real),&Arg0) == Error) 
         RunErr(0, NULL);
      }
   Return;
   }

/*
 * dtor(x), x in degrees
 */

FncDcl(dtor,1)
   {
   union numeric r;

   if (cvreal(&Arg1, (double *)&r) == CvtFail) 
      RunErr(102, &Arg1);
   if (mkreal(r.real * PI / 180, &Arg0) == Error) 
      RunErr(0, NULL);
   Return;
   }

/*
 * rtod(x), x in radians
 */
FncDcl(rtod,1)
   {
   union numeric r;

   if (cvreal(&Arg1, (double *)&r) == CvtFail) 
      RunErr(102, &Arg1);
   if (mkreal(r.real * 180 / PI, &Arg0) == Error) 
      RunErr(0, NULL);
   Return;
   }

/*
 * exp(x)
 */

FncDcl(exp,1)
   {
   int t;
   double y;
   union numeric r;
   double exp();

   if ((t = cvreal(&Arg1, (double *)&r)) == CvtFail) 
      RunErr(102, &Arg1);
   errno = 0;
   y = exp(r.real);
   if (errno == ERANGE) 
      RunErr(-204, NULL);
   if (mkreal(y,&Arg0) == Error) 
      RunErr(0, NULL);
   Return;
   }

/*
 * log(x,b) - logarithm of x to base b.
 */
FncDcl(log,2)
   {
   static double lastbase = 0.0;
   static double divisor;
   double x, b;
   double log();
 
   if (cvreal(&Arg1, &x) != T_Real)
      RunErr(102, &Arg1);
   if (x <= 0.0)
      RunErr(205, &Arg1);
   x = log(x);
   if (! ChkNull(Arg2))  {
      if (cvreal(&Arg2, &b) != T_Real)
         RunErr(102, &Arg2);
      if (b <= 1.0)
         RunErr(205, &Arg2);
      if (b != lastbase) {
         divisor = log(b);
         lastbase = b;
         }
      x = x / divisor;
      }  
   if (mkreal(x, &Arg0) == Error)
      RunErr(0, NULL);
   Return;
   }


/*
 * sqrt(x)
 */

FncDcl(sqrt,1)
   {
   int t;
   double y;
   union numeric r;
   double sqrt();

   if ((t = cvreal(&Arg1, (double *)&r)) == CvtFail) 
      RunErr(102, &Arg1);
   y = sqrt(r.real);
   errno = 0;
   if (errno == EDOM) 
      RunErr(-205, NULL);
   if (mkreal(y,&Arg0) == Error) 
      RunErr(0, NULL);
   Return;
   }
#else					/* MathFncs */
static char x;			/* prevent empty module */
#endif					/* MathFncs */
